/* eslint-disable no-unused-vars */
import applySpec from 'ramda/src/applySpec';
import compose from 'ramda/src/compose';
import find from 'ramda/src/find';
import pathOr from 'ramda/src/pathOr';
import prop from 'ramda/src/prop';
import propEq from 'ramda/src/propEq';

import { parseJsonString } from './base-util';

export const extractImage = (key: string = 'image') =>
    compose(pathOr('', ['image', '0', 'selected', 'url']), parseJsonString, pathOr('', [key]));

export const resolveImage = (imageKey = 'image', altKey = 'image_alt_text') =>
    applySpec({
        src: extractImage(imageKey),
        alt: getAltText(imageKey, altKey),
    });

/** alt text */
const getBynderAltText = (propKey: string) =>
    compose(pathOr('', ['value']), find(propEq('SEO_Tags', 'name')), pathOr([], [propKey, 0, 'textMetaproperties']));

export const extractBynderDescription = (key: string = 'image') =>
    compose(pathOr('', ['image', '0', 'description']), parseJsonString, pathOr('', [key]));

export function getAltText(propKey = 'image', defaultPropKey = 'image_alt_text') {
    return function (obj: any) {
        return prop(defaultPropKey)(obj) || getBynderAltText(propKey)(obj) || extractBynderDescription(propKey);
    };
}

/** end alt text */

/** DAT specific */

export const getBasicDATWidthHeight = (width: number = 900, height: number = 800, transform = 'scale') =>
    `io=transform:${transform},width:${width},height:${height}`;

/** types */
export type DATOptions = { width?: number; height?: number; transform: 'fill' | 'scale' };
export type BynderImageDataType = {
    /** selected is manually added, using additionalInfo from bynder */
    selected?: {
        url: string;
        fileSize: number | null;
        width: number | null;
        height: number | null;
        imageType?: string;
        type: 'VIDEO' | 'IMAGE';
        altText?: string;
        fit: 'cover' | 'contain' | 'scale-down' | 'none';
        position: string;
        bgColor: string;
    };
    isPublic: boolean;
    __typename?: 'Image';
    id: string;
    name: string;
    description: string | null;
    databaseId: string;
    originalUrl?: string;
    type: 'VIDEO' | 'IMAGE' | 'DOCUMENT';
    url?: string;
    previewUrls?: string[];
    extensions?: string[];
    textMetaproperties?: ({
        name: string;
        value: string;
    } | null)[];
    files: {
        [key: string]: {
            url: string;
            width: number | null;
            height: number | null;
            fileSize: number | null;
        };
    };
};
/** determine whether the image is
     * 1. custom transformation ('customTransformation') ==> further transformation cannot be applied
    https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1?io=overlay:text,content:abc,font:Open Sans,size:64,color:FFFFFFFF,gravity:center,padding:20,angle:0

    * 2. base transform url ('transformBaseUrl') ==> transformation can be applied in the component
 * "https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1"

* 3. bynder defined preset ('preset') ==> further transformation cannot be applied
    "https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1"

    * 4. custom preset ('defaultPreset') example: thumbnail, small, mini, original , webImage, ==> further transformation can be applied

    * returns either 'customTransformation' | 'transformBaseUrl' | or preset type of the image example webImage, thumbnail
    * */
export const getImageType = (url: string, fileName: string, files: Record<string, any>) => {
    if (!url) return '';
    if (url.includes(`${fileName}?io=`)) {
        return ImageType.customTransformation;
    } else {
        const parts = url.split('/');
        if (parts[3] === 'transform' && parts[5] === fileName) {
            // url?.toLowerCase().includes(`transform/${databaseId}/${fileName}`.toLowerCase()) this check creates error in sometimes url does not have - which is present in database id

            return ImageType.transformBaseUrl;
        } else {
            return retrieveImageKey(files, url);
        }
    }
};
export const retrieveImageKey = (files: Record<string, any>, url: string = '') => {
    for (const key in files) {
        if (files[key].url === url) {
            return key;
        }
    }
    return ImageType.defaultPreset;
};

export enum ImageType {
    defaultPreset = 'defaultPreset',
    customTransformation = 'customTransformation',
    transformBaseUrl = 'transformBaseUrl',
}

export const getUrlBasedOnDerivative = (url: string, imageDerivativeType: string, suffix: string) => {
    if (imageDerivativeType === ImageType.customTransformation) {
        return `${url}&${suffix}`;
    } else if (imageDerivativeType === ImageType.transformBaseUrl) {
        return `${url}?${suffix}`;
    }
    return url;
};

/** end DAT */

/**
 *
 * @param selected : object containing select image url
 * @param isPublic : whether asset is marked public, if it is video and marked public, then mov or gif can be selected otherwise, only webImage can be selected in that case we have to show previewUrl
 * @returns returns selected video or image, if asset is video and is not public, then returns previewUrl
 */
export function getMediaUrl(
    selected: BynderImageDataType['selected'],
    isPublic: boolean,
    previewUrls: BynderImageDataType['previewUrls']
): string {
    const { url = '', type: fileType = '' } = selected ?? {};
    if (fileType === 'VIDEO') {
        if (isPublic) {
            return url;
        } else {
            return previewUrls?.[0] || '';
        }
    }
    return url;
}
